//SerialPort.cpp

/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPFserial is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/
/*
	Current Author: Marty Kraimer, Thomas Coleman
	Original Author: Jim Kowalkowski (for Hideos)
	Date: 09JUL98
*/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <epicsEvent.h>
 
#include "DLList.h"
#include "SerialPort.h"
 
static DLList *pSerialPortList = 0;

SerialPort:: SerialPort()
: byteHandler(0), byteHandlerParm(0), sniffHandler(0), sniffHandlerParm(0),
  isBound(false), portName(0),
  firstFlush(true), firstConfig(true), firstGetConfig(true)
{}

int SerialPort::registerName(const char *name)
{
    if(portName!=0) {
        printf("SerialPort::registerName already registered %s\n",name);
        return(-1);
    }
    portName = name;
    if(pSerialPortList==0) pSerialPortList = new DLList;
    DLNode *node = DLList::allocNode();
    node->obj = this;
    pSerialPortList->addEnd(node);
    return(0);
}

SerialPort *SerialPort::find(const char *portName)
{
    if(pSerialPortList==0) pSerialPortList = new DLList;
    DLNode *node = pSerialPortList->getFirst();
    while(node) {
        SerialPort *pSerialPort = (SerialPort *)node->obj;
        if(::strcmp(portName,pSerialPort->portName)==0) {
            return(pSerialPort);
        }
        node = pSerialPortList->getNext(node);
    }
    return(0);
}

SerialPort *SerialPort::bind(const char *portName,void* parm,BYTE_HANDLER b)
{
    SerialPort *pSerialPort = find(portName);
    if(!pSerialPort) {
	printf("%s port not found\n",portName);
	return(0);
    }
    if(pSerialPort->isBound) {
        printf("%s port already bound\n",portName);
        return(0);
    }
    if(b==0) {
	printf("%s bind call but no byte handler supplied",portName);
	return(0);
    }
    pSerialPort->byteHandlerParm=parm;
    pSerialPort->byteHandler=b;
    pSerialPort->isBound = true;
    return(pSerialPort);
}

void SerialPort::release()
{
    byteHandler=0; byteHandlerParm=0; isBound = false;
}

bool SerialPort::setSniffHandler(
    const char *portName,void* sniffHandlerParm,SNIFF_HANDLER sniffHandler)
{
    SerialPort *pSerialPort = find(portName);
    if(!pSerialPort) {
	printf("%s port not found\n",portName);
	return(false);
    }
    if(!pSerialPort->isBound) {
        printf("%s port not bound\n",portName);
        return(false);
    }
    if(pSerialPort->sniffHandler!=0) {
	printf("%s already has a SNIFF_HANDLER\n",portName);
	return(0);
    }
    pSerialPort->sniffHandlerParm = sniffHandlerParm;
    pSerialPort->sniffHandler = sniffHandler;
    return(true);
}

void SerialPort::freeSniffHandler(const char *portName)
{
    SerialPort *pSerialPort = find(portName);
    if(!pSerialPort) {
	printf("%s port not found\n",portName);
	return;
    }
    pSerialPort->sniffHandler = 0;
    pSerialPort->sniffHandlerParm = 0;
    return;
}

void SerialPort::flush()
{
    if(firstFlush) printf("%s flush is not available\n",portName);
    firstFlush = false;
}

bool SerialPort::config( int , int , int , char , char )
{
    if(firstConfig) printf("%s config is not available\n",portName);
    firstConfig = false;
    return(false);
} 
void SerialPort::getConfig( int *, int *, int *, char *, char *)
{
    if(firstGetConfig) printf("%s getConfig is not available\n",portName);
    firstGetConfig = false;
} 
