//serialPortSniff.cc

/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPFserial is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/
/*
	Author: Marty Kraimer
	Date: 21OCT98
*/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include <epicsRingBytes.h>
#include <epicsThread.h>
#include <epicsTime.h>
#include <cantProceed.h>
 
#include "SerialPort.h"
 
class SniffData {
public:
    bool isReceive;
    unsigned char byte;
};

static int ringBufSize = 1000 * sizeof(SniffData);

class SniffPrivate {
public:
    epicsRingBytesId ringbuf;
};

typedef struct peakInfo{
    epicsThreadId threadid;
    const char *portName;
    int nseconds;
}peakInfo;

static void peakThread(peakInfo *ppeakInfo);

static char threadname[] = "serialSniff";

extern "C" int serialPortSniff(const char *portName,int nseconds)
{
    peakInfo *ppeakInfo;

    ppeakInfo = (peakInfo *)callocMustSucceed(
        1,sizeof(peakInfo),"serialPortSniff");
    ppeakInfo->portName = portName;
    ppeakInfo->nseconds = nseconds;
    ppeakInfo->threadid = epicsThreadCreate(
        threadname,epicsThreadPriorityLow,
        epicsThreadGetStackSize(epicsThreadStackSmall),
        (EPICSTHREADFUNC)peakThread,(void *)ppeakInfo);
    return(0);
}

static void peakHandler(void* parm,bool isReceive, unsigned char byte)
{
    SniffPrivate *pvt = (SniffPrivate *)parm;
    SniffData data;
    data.isReceive = isReceive;
    data.byte = byte;
    epicsRingBytesPut(pvt->ringbuf,(char *)&data,sizeof(SniffData));
}

static void printQueue(bool *isReceiveQueue, unsigned char *byteQueue,int len)
{
    for(int i=0;i<len; i++) 
	printf("%c%2.2x ",(isReceiveQueue[i] ? 'R' : 'T'),byteQueue[i]);
    for(int i=0;i<len; i++)
	printf("%c", (isprint(byteQueue[i]) ? byteQueue[i] : '.'));
    printf("\n");
}
    
static void peakThread(peakInfo *ppeakInfo)
{
    const char *portName = ppeakInfo->portName;
    double nseconds = ppeakInfo->nseconds;
    SniffPrivate pvt;
    bool isReceiveQueue[10];
    unsigned char byteQueue[10];
    int nextQueue = 0;

    if((pvt.ringbuf = epicsRingBytesCreate(sizeof(void *) * ringBufSize))==NULL){
	printf("peakThread: initWatchDog failed\n");
        free(ppeakInfo);
	return;
    }
    if(!SerialPort::setSniffHandler(portName,&pvt,peakHandler)) return;
    epicsTimeStamp startTime,currentTime;
    epicsTimeGetCurrent(&startTime);
    printf("\n");
    while(true) {
        epicsThreadSleep(1.0);
	while (epicsRingBytesUsedBytes(pvt.ringbuf)>=(int)sizeof(SniffData)){
	    SniffData data;
	    if(epicsRingBytesGet(pvt.ringbuf,(char *)&data,
	    sizeof(SniffData))!=(int)sizeof(SniffData)) {
		printf("peakThread epicsRingBytesGet returned error\n");
	    } else {
		isReceiveQueue[nextQueue] = data.isReceive;
		byteQueue[nextQueue] = data.byte;
		if(++nextQueue == 10) {
		    printQueue(isReceiveQueue,byteQueue,nextQueue);
		    nextQueue = 0;
		}
	    }
            epicsTimeGetCurrent(&currentTime);
            if(epicsTimeDiffInSeconds(&currentTime,&startTime)>=nseconds) break;
	}
        epicsTimeGetCurrent(&currentTime);
        if(epicsTimeDiffInSeconds(&currentTime,&startTime)>=nseconds) break;
    }
    if(nextQueue>0) printQueue(isReceiveQueue,byteQueue,nextQueue);
    printf("serialPortSniff done\n");
    SerialPort::freeSniffHandler(portName);
    epicsRingBytesDelete(pvt.ringbuf);
    free(ppeakInfo);
}

#include <iocsh.h>
extern "C" {
static const iocshArg serialPortSniffArg0 = { "portName",iocshArgString};
static const iocshArg serialPortSniffArg1 = { "seconds",iocshArgInt};
static const iocshArg * const serialPortSniffArgs[2] = {
    &serialPortSniffArg0,&serialPortSniffArg1};
static const iocshFuncDef serialPortSniffFuncDef = {
    "serialPortSniff",2,serialPortSniffArgs};
static void serialPortSniffCallFunc(const iocshArgBuf *args)
{
    serialPortSniff(args[0].sval,args[1].ival);
}

static void serialPortSniffRegister(void)
{
    iocshRegister(&serialPortSniffFuncDef,serialPortSniffCallFunc);
}
} // extern "C"

class SniffRegister {
public:
    SniffRegister() {serialPortSniffRegister();}
};

static SniffRegister serialPortSniffRegisterObj;

